﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

//
// Sample API connection to www.tradingphysics.com API Server
// (c) 2009-2011 www.tradingphysics.com
// By using this program you agree to the terms 
// http://www.tradingphysics.com/Terms.aspx
//

namespace SampleAPI
{
    class Program
    {
        static String ServerUrl = "http://api.tradingphysics.com/";

        // data query parameters
        static String[] TickTimes = { "1000", "5000", "60000" }; // tick times of 1 sec, 5 secs, and 1 minute

        static String[] DataTypes = { "timesales", "orderflow", "indicators", "oscillators" }; // type of data 

        static String[] DataFormats = { "CSV", "XML", "TXT" }; // formats of data

        static String[] CompressionTypes = { "stream", /* compress HTTP response stream */
                                             "auto", /* compress HTTP response stream only if the browser supports it, otherwise compress file */
                                             "file", /* return file compressed in .z8 format */
                                             "none"}; /* no compression */
       
        static String Date = "20090803"; // target date (use today's date for real-time streams) in YYYYMMDD string format */
        static String Stock = "QQQQ"; // target stock ticker

        // custom indicator settings
        static String Indicators = "MarketIndicator.Buy.Executed.VolumeA+MarketIndicator.Sell.Executed.VolumeA";

        // query to get the data
        static String GetDataQuery = "getdata?type={0}&date={1}&stock={2}&format={3}&compression={4}";

        // query to get the ticket for the data query
        static String GetTicketQuery = "getticket?C={0}&P={1}?{2}"; 

        // initialize with valid account information
        // to receive free evaluation credits, go to https://www.tradingphysics.com/User/Register.aspx
        // this information is available on your Account Information Page
        static String CustomerNumber = "";
        static String PasswordHash = "";

      
        static void Main(string[] args)
        {
            // build query to retrieve time and sales data in CSV format without compression
            String DataQuery = String.Format(GetDataQuery, 
                DataTypes[0], /* time&sales */
                Date, 
                Stock, 
                DataFormats[0], /* CSV format */
                CompressionTypes[3] /* no compression */);

            // get ticket to authorize the data retrieval using the above query,
            // the ticket will be obtained using the customer number and password hash
            // the server will refuse payable data retrival if the ticket is not available/invalid
            String TicketQuery = String.Format(GetTicketQuery, CustomerNumber, PasswordHash, DataQuery);

            // the ticket will go here
            String Ticket = "";

            // request ticket first
            HttpWebRequest r = (HttpWebRequest)WebRequest.Create(ServerUrl + TicketQuery);
            try
            {
                HttpWebResponse rr = (HttpWebResponse)r.GetResponse();
                if (rr.StatusCode != HttpStatusCode.OK)
                {
                    System.Console.WriteLine("HTTP Error: " + rr.StatusDescription);
                    return;
                }

                // get ticket from response stream if transaction was successful
                StreamReader Reader = new StreamReader(rr.GetResponseStream());
                Ticket = Reader.ReadToEnd();
            }
            catch (WebException e)
            {
                // bad request
                System.Console.WriteLine("Web Exception: " + e.Message);

                // read error description from response stream
                if (e.Response != null)
                {
                    StreamReader Reader = new StreamReader(e.Response.GetResponseStream());
                    System.Console.WriteLine("Description: " + Reader.ReadToEnd());
                }
                return;
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Exception: " + e.Message);
                return;
            }

            // print ticket
            System.Console.WriteLine("Ticket: " + Ticket);

            // update query to include the ticket (ticket may be an empty string if no ticket required for this request)
            if (Ticket != "") DataQuery += "&t=" + Ticket;
              
            // request the data itself
            // if transaction is payable, the credits will be charged only upon first ticket usage
            // the url with the same ticket may be reused without additional charges
            r = (HttpWebRequest)WebRequest.Create(ServerUrl + DataQuery);
            try
            {
                HttpWebResponse rr = (HttpWebResponse)r.GetResponse();
                if (rr.StatusCode != HttpStatusCode.OK)
                {
                    System.Console.WriteLine("HTTP Error: " + rr.StatusDescription);
                    return;
                }

                // get data from response stream if transaction was successful
                // we assume that the data was retrieved in text format so that we can print it on the console window
                System.Console.WriteLine("--- Begin stream ---");
                StreamReader Reader = new StreamReader(rr.GetResponseStream());
                while (!Reader.EndOfStream)
                {
                    System.Console.WriteLine(Reader.ReadLine());
                }
                System.Console.WriteLine("--- End stream ---");
            }
            catch (WebException e)
            {
                // bad request
                System.Console.WriteLine("Web Exception: " + e.Message);
                // read error description from response stream
                if (e.Response != null)
                {
                    StreamReader Reader = new StreamReader(e.Response.GetResponseStream());
                    System.Console.WriteLine("Description: " + Reader.ReadToEnd());
                }
                return;
            }
            catch (Exception e)
            {
                System.Console.WriteLine("Exception: " + e.Message);
                return;
            }

            return;
        }
    }
}
